% Copyright (c) Camden Dixie O'Brien
% SPDX-License-Identifier: AGPL-3.0-only

-module(proto_sup).
-behaviour(supervisor).

-export([start_link/2]).
-export([init/1]).

start_link(Port, CertDir) ->
	supervisor:start_link({local, ?MODULE}, ?MODULE, [Port, CertDir]).

init([Port, CertDir]) ->
	SupFlags = #{strategy => one_for_one,
				 intensity => 5,
				 period => 10},
	ChildSpecs = [#{id => session_sup,
					start => {session_sup, start_link, []},
					restart => permanent,
					shutdown => 5000,
					type => supervisor,
					modules => [session_sup]},
				  #{id => tcp_server,
					start => {tcp_server, start_link, [Port, CertDir]},
					restart => permanent,
					shutdown => 5000,
					type => worker,
					modules => [tcp_server]}],
	{ok, {SupFlags, ChildSpecs}}.
